@@ -7,55 +7,31 @@ use warnings;
 use Exporter 'import';
 
 our @EXPORT_OK = qw(read_sxc read_sxc_fh read_xml_file read_xml_string);
-our $VERSION = '0.23';
+our $VERSION = '0.25';
 
 use Archive::Zip ':ERROR_CODES';
-use XML::Parser;
 use Carp qw(croak);
+use Spreadsheet::ParseODS;
+use PerlX::Maybe;
 
-my %workbook = ();
-my @worksheets = ();
-my @sheet_order = ();
-my $table = "";
-my $row = -1;
-my $col = -1;
-my $text_p = -1;
-my @cell = ();
-my $repeat_cells = 1;
-my $repeat_rows = 1;
-my $row_hidden = 0;
-my $date_value = '';
-my $time_value = '';
-my $currency_value = '';
-my $max_datarow = -1;
-my $max_datacol = -1;
-my $col_count = -1;
-my @hidden_cols = ();
-my %options = ();
+our @CARP_NOT = qw(Spreadsheet::ParseODS);
 
 sub zip_error_handler {}
 
 sub read_sxc ($;$) {
     my ($sxc_file, $options_ref) = @_;
-    if( !$options_ref->{StrictErrors}) {
-        -f $sxc_file && -s _ or return undef;
-    };
-    open my $fh, '<', $sxc_file
-        or croak "Couldn't open '$sxc_file': $!";
-    read_sxc_fh( $fh, $options_ref );
+    #if( !$options_ref->{StrictErrors}) {
+    #    -f $sxc_file && -s _ or return undef;
+    #};
+    #open my $fh, '<', $sxc_file
+    #    or croak "Couldn't open '$sxc_file': $!";
+    #read_sxc_fh( $fh, $options_ref );
+    _parse_xml( {}, $options_ref, $sxc_file );
 }
 
 sub read_sxc_fh {
-    my ($fh, $options_ref) = @_;
-    my $zip = Archive::Zip->new();
-    my $status = $zip->readFromFileHandle($fh);
-    $status == AZ_OK
-        or croak "Read error from zip";
-    my $content = $zip->memberNamed('content.xml');
-    $content->rewindData();
-    my $stream = $content->fh;
-    binmode $stream => ':gzip(none)';
-    _parse_xml( {}, $stream, $options_ref );
+    my( $stream, $options_ref ) = @_;
+    _parse_xml( {},  $options_ref, $stream );
 }
 
 sub read_xml_file ($;$) {
@@ -63,209 +39,105 @@ sub read_xml_file ($;$) {
     if( !$options_ref->{StrictErrors}) {
         -f $xml_file && -s _ or return undef;
     };
-    _parse_xml({ method => 'parsefile' }, $xml_file, $options_ref);
+    _parse_xml({ method => 'parsefile', type => 'xml' }, $options_ref, $xml_file);
 }
 
 sub read_xml_string ($;$) {
     my ($xml_string, $options_ref) = @_;
-    _parse_xml( {}, $xml_string, $options_ref );
+    _parse_xml( { type => 'xml' }, $options_ref, \$xml_string );
 }
 
 sub _parse_xml {
-    my ($internal_options, $xml_thing, $options_ref) = @_;
-    %workbook = ();
-    @worksheets = ();
-    if ( defined $options_ref ) { %options = %{$options_ref}};
-    my $parse = $internal_options->{ method } || 'parse';
-    my $p = XML::Parser->new(Handlers => {Start => \&handle_start,
-                          End => \&handle_end,
-                          Char => \&char_start});
-    $p->$parse($xml_thing);
-    if ( $options{OrderBySheet} ) { return [@worksheets] } else { return {%workbook} }
-}
+    my ($internal_options, $options_ref, $xml_thing) = @_;
 
-sub handle_start {
-    my ($expat, $element, %attributes) = @_;
-    if ( $element eq "text:p" ) {
-# increase paragraph count if not part of an annotation
-        if ( ! $expat->within_element('office:annotation') ) { $text_p++; }
-    }
-    elsif ( ( $element eq "table:table-cell" ) or ( $element eq "table:covered-table-cell" ) ) {
-# increase cell count
-        $col++;
-# if number-columns-repeated is set, set $repeat_cells value accordingly for later use
-        if ( exists $attributes{'table:number-columns-repeated'} ) {
-            $repeat_cells = $attributes{'table:number-columns-repeated'};
-        }
-# save the currency value (if available)
-        if (exists $attributes{'table:value'} or exists $attributes{'office:value'} ) {
-            $currency_value = $attributes{'table:value'} || $attributes{'office:value'};
-        }
-# if cell contains date or time values, set boolean variable for later use
-        elsif (exists $attributes{'table:date-value'} or exists $attributes{'office:date-value'}) {
-            $date_value = $attributes{'table:date-value'} || $attributes{'office:date-value'};
-        }
-        elsif (exists $attributes{'table:time-value'} or exists $attributes{'office:time-value'}) {
-            $time_value = $attributes{'table:time-value'} || $attributes{'office:time-value'};
-        }
-    }
-    elsif ( $element eq "table:table-row" ) {
-# increase row count
-        $row++;
-# if row is hidden, set $row_hidden for later use
-        if ( exists $attributes{'table:visibility'} ) { $row_hidden = 1 } else { $row_hidden = 0 }
-# if number-rows-repeated is set, set $repeat_rows value accordingly for later use
-        if ( exists $attributes{'table:number-rows-repeated'} ) {
-            $repeat_rows = $attributes{'table:number-rows-repeated'};
-        }
-    }
-    elsif ( $element eq "table:table-column" ) {
-# increase column count
-        $col_count++;
-# if columns is hidden, add column number to @hidden_cols array for later use
-        if ( exists $attributes{'table:visibility'} ) {
-            push @hidden_cols, $col_count;
-        }
-# if number-columns-repeated is set and column is hidden, add affected columns to @hidden_cols
-        if ( exists $attributes{'table:number-columns-repeated'} ) {
-            $col_count++;
-            if ( exists $attributes{'table:visibility'} ) {
-                for (2..$attributes{'table:number-columns-repeated'}) {
-                    push @hidden_cols, $hidden_cols[$#hidden_cols] + 1;
-                }
-            }
-        }
-    }
-    elsif ( $element eq "table:table" ) {
-# get name of current table
-        $table = $attributes{'table:name'};
-    }
-}
+    $options_ref ||= {};
 
-sub handle_end {
-    my ($expat, $element) = @_;
-    if ( $element eq "text:p" ) {
-        if( ! $expat->within_element('office:annotation') ) {
-            $cell[ $text_p ] ||= undef;
-        };
-    }
-    elsif ( $element eq "table:table") {
-# decrease $max_datacol if hidden columns within range
-        if ( ( ! $options{NoTruncate} ) and ( $options{DropHiddenColumns} ) ) {
-            for ( 1..scalar grep { $_ <= $max_datacol } @hidden_cols ) {
-                $max_datacol--;
-            }
-        }
-# truncate table to $max_datarow and $max_datacol
-        if ( ! $options{NoTruncate} ) {
-            $#{$workbook{$table}} = $max_datarow;
-            foreach ( @{$workbook{$table}} ) {
-                $#{$_} = $max_datacol;
-            }
-        }
-# set up alternative data structure
-        if ( $options{OrderBySheet} ) {
-            push @worksheets, (
-                {
-                    label   => $table,
-                    data    => \@{$workbook{$table}},
-                }
-            );
-        }
-# reset table, column, and row values to default for next table
-        $row = -1;
-        $max_datarow = -1;
-        $max_datacol = -1;
-        $table = "";
-        $col_count = -1;
-        @hidden_cols = ();
-    }
-    elsif ( $element eq "table:table-row" ) {
-# drop hidden columns from current row
-        if ( $options{DropHiddenColumns} ) {
-            foreach ( reverse @hidden_cols ) {
-                splice @{$workbook{$table}[$row]}, $_, 1;
-            }
-        }
-# drop current row, if hidden
-        if ( ( $options{DropHiddenRows} ) and ( $row_hidden == 1 ) ) {
-            pop @{$workbook{$table}};
-            $row--;
-        }
-# repeat current row, if necessary
-        else {
-            for (2..$repeat_rows) {
-                $row++;
-                $workbook{$table}[$row] = $workbook{$table}[$row - 1]   # copy reference, not data
-            }
-# set max_datarow, if row not empty
-            if ( grep { defined $_ } @{$workbook{$table}[$row]} ) {
-                $max_datarow = $row;
-            }
-        }
-# reset row and col values to default for next row
-        $repeat_rows = 1;
-        $col = -1;
-    }
-    elsif ( ( $element eq "table:table-cell" ) or ( $element eq "table:covered-table-cell" ) ) {
-# assign currency, date or time value to current workbook cell if requested
-        if ( ( $options{StandardCurrency} ) and ( length( $currency_value ) ) ) {
-            $workbook{$table}[$row][$col] = $currency_value;
-            $currency_value = '';
+    my $type;
+    if( $internal_options->{ type } ) {
+        $type = $internal_options->{ type };
+    };
 
-        }
-        elsif ( ( $options{StandardDate} ) and ( $date_value ) ) {
-            $workbook{$table}[$row][$col] = $date_value;
-            $date_value = '';
-        }
-        elsif ( ( $options{StandardTime} ) and ( $time_value ) ) {
-            $workbook{$table}[$row][$col] = $time_value;
-            $time_value = '';
-        }
-# join cell contents and assign to current workbook cell
-        else {
-            $workbook{$table}[$row][$col] = @cell ? join $options{ReplaceNewlineWith} || "",
-                map { defined($_) ? $_ : '' } @cell : undef;
-        }
-# repeat current cell, if necessary
-        for (2..$repeat_cells) {
-            $col++;
-            $workbook{$table}[$row][$col] = $workbook{$table}[$row][$col - 1];
-        }
-# reset cell and paragraph values to default for next cell
-        @cell = ();
-        $repeat_cells = 1;
-        $text_p = -1;
-    }
-}
+    my $line_sep = $options_ref->{ReplaceNewlineWith} || "";
 
-sub char_start {
-    my ($expat, $content) = @_;
-# don't include paragraph if part of an annotation
-    if ( $expat->within_element('office:annotation') ) {
-        return;
-    }
-# don't include covered cells, if not requested
-    if ( ( $expat->within_element('table:covered-table-cell') ) and ( ! $options{IncludeCoveredCells} ) ) {
-        return;
-    }
-# add paragraph or textspan to current @cell array
-    if ( $table ) {
-        $cell[$text_p] .= $content;
-# set $max_datarow and $max_datacol to current values
-        $max_datarow = $row;
-        if ( $col > $max_datacol ) { $max_datacol = $col }
+    my $workbook;
+    my $ok = eval {
+        $workbook = Spreadsheet::ParseODS->new(
+                       line_separator => $line_sep,
+                        #%$options_ref,
+                    )->parse( $xml_thing,
+                              maybe inputtype => $type );
+        1;
+   };
+   if( my $err = $@ and $options_ref->{ StrictErrors } ) {
+       die "$@\n"
+   };
+   return unless $workbook;
+
+    # Cut off trailing columns if any
+    # Down-convert from ::Cell to raw values, depending on the options
+
+    # This conversion is likely best done in an immediate callback to
+    # speed up things
+    my $res = {};
+    for my $s ($workbook->worksheets) {
+        my $rs = $res->{ $s->label } = [];
+        for my $r ($s->row_min..$s->row_max) {
+            if( $options_ref->{DropHiddenRows} and $s->is_row_hidden( $r ) ) {
+                next;
+            };
+
+            my $rowref;
+            for my $c ($s->col_min..$s->col_max) {
+                # Depending on what type we want, use ->value or ->unformatted
+                # depending on $options_ref->{ ... }
+                if( ! $rowref ) {
+                    push @$rs, ($rowref = []);
+                };
+
+                my $cell = $s->get_cell( $r,$c );
+                if( $options_ref->{DropHiddenColumns} and $s->is_col_hidden($c)) {
+                    next;
+                };
+
+                my ($method,$type) = ('value',$cell->type);
+                $type ||= '';
+
+                if( $options_ref->{StandardCurrency} and $type =~ qr/^(float|currency|percentage)/) {
+                    $method = 'unformatted';
+                };
+                if( $options_ref->{StandardDate} and $type =~ qr/^(date)/) {
+                    $method = 'unformatted';
+                };
+                if( $options_ref->{StandardTime} and $type =~ qr/^(time)/) {
+                    $method = 'unformatted';
+                };
+
+                push @$rowref, $s->get_cell( $r,$c )->$method;
+            }
+        }
+    };
+
+    if ( $options_ref->{OrderBySheet} ) {
+        return [ map { $res->{ $_ } } $workbook->worksheets ]
+    } else {
+        return $res
     }
 }
 
 1;
+
 __END__
 =head1 NAME
 
 Spreadsheet::ReadSXC - Extract OpenOffice 1.x spreadsheet data
 
-=head1 SYNOPSIS
+=head1 NOTICE
 
+This is a legacy API wrapper. Most likely you want to look at
+L<Spreadsheet::ParseODS>, which implements an API more compatible with
+L<Spreadsheet::ParseXLSX>. That module is also the backend for this API.
+
+=head1 SYNOPSIS
 
   use Spreadsheet::ReadSXC qw(read_sxc);
   my $workbook_ref = read_sxc("/path/to/file.sxc");
@@ -579,10 +451,8 @@ Parses an XML string and eturns the worksheets as a data structure.
     my $fh = fetch_url('http://example.com/example.ods');
     my $sheet = read_sxc_fh( $fh );
 
-
 =head1 SEE ALSO
 
-
 L<https://www.openoffice.org/xml/general.html> has extensive documentation
 of the OpenOffice 1.x XML file format (soon to be replaced by the
 OASIS file format (ODS), see L<http://docs.oasis-open.org/office/v1.2/OpenDocument-v1.2.pdf>).
